package de.intevation.flys.exports;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;

import org.w3c.dom.Document;

import org.apache.log4j.Logger;

import au.com.bytecode.opencsv.CSVWriter;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;


/**
 * An abstract exporter that implements some basic methods for exporting data of
 * artifacts.
 *
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public abstract class AbstractExporter implements OutGenerator {

    /** The logger used in this exporter.*/
    private static Logger logger = Logger.getLogger(AbstractExporter.class);


    /** The name of the CSV facet which triggers the CSV creation.*/
    public static final String FACET_CSV = "csv";

    /** The default charset for the CSV export.*/
    public static final String DEFAULT_CSV_CHARSET = "UTF-8";

    /** The default separator for the CSV export.*/
    public static final char DEFAULT_CSV_SEPARATOR = ',';


    /** The document of the incoming out() request.*/
    protected Document request;

    /** The output stream where the data should be written to.*/
    protected OutputStream out;

    /** The CallContext object.*/
    protected CallContext context;

    /** The selected facet.*/
    protected String facet;

    /** The master artifact.*/
    protected Artifact master;


    /**
     * Concrete subclasses need to use this method to write their special data
     * objects into the CSV document.
     *
     * @param writer The CSVWriter.
     */
    protected abstract void writeCSVData(CSVWriter writer);


    /**
     * This method enables concrete subclasses to collected its own special
     * data.
     *
     * @param artifacts The artifact that stores the data that has to be
     * exported.
     */
    protected abstract void addData(Artifact artifact);


    public void init(Document request, OutputStream out, CallContext context) {
        logger.debug("AbstractExporter.init");

        this.request = request;
        this.out     = out;
        this.context = context;
    }


    public void setMasterArtifact(Artifact master) {
        this.master = master;
    }


    /**
     * This doOut() just collects the data of multiple artifacts. Therefore, it
     * makes use of the addData() method which enables concrete subclasses to
     * store its data on its own. The real output creation takes place in the
     * concrete generate() methods.
     *
     * @param artifact The artifact.
     * @param facet The facet to add - NOTE: the facet needs to fit to the first
     * facet inserted into this exporter. Otherwise this artifact/facet is
     * skipped.
     * @param attr The attr document.
     */
    public void doOut(Artifact artifact, String facet, Document attr) {
        logger.debug("AbstractExporter.doOut: " + facet);

        if (!isFacetValid(facet)) {
            logger.warn("Facet '" + facet + "' not valid. No output created!");
            return;
        }

        addData(artifact);
    }


    /**
     * Generates an export based on a specified facet.
     */
    public void generate()
    throws IOException
    {
        logger.debug("AbstractExporter.generate");

        if (facet != null && facet.equals(FACET_CSV)) {
            generateCSV();
        }
        else {
            throw new IOException("invalid facet for exporter.");
        }
    }


    /**
     * Determines if the desired facet is valid for this exporter. If no facet
     * is currently set, <i>facet</i> is set.
     *
     * @param facet The desired facet.
     *
     * @return true, if <i>facet</i> is valid, otherwise false.
     */
    protected boolean isFacetValid(String facet) {
        logger.debug("AbstractExporter.isFacetValid");

        if (facet == null || facet.length() == 0) {
            return false;
        }
        else if (this.facet == null || this.facet.length() == 0) {
            logger.debug("Set the facet of this export: " + facet);
            this.facet = facet;

            return true;
        }
        else {
            return this.facet.equals(facet);
        }
    }


    /**
     * This method starts CSV creation. It makes use of writeCSVData() which has
     * to be implemented by concrete subclasses.
     */
    protected void generateCSV()
    throws    IOException
    {
        logger.info("AbstractExporter.generateCSV");

        CSVWriter writer = new CSVWriter(
            new OutputStreamWriter(
                out,
                DEFAULT_CSV_CHARSET),
            DEFAULT_CSV_SEPARATOR);

        writeCSVData(writer);

        writer.close();
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
